# bzr-dbus: dbus support for bzr/bzrlib.
# Copyright (C) 2007 Canonical Limited.
#   Author: Robert Collins.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 of the License.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
# 

"""System wide hooks to trigger dbus events from bzr activity."""


from bzrlib.branch import Branch
from bzrlib.smart.server import SmartTCPServer
from bzrlib.trace import warning

import activity
from server_mainloop import (
    MainloopThread, install_hooks as install_mainloop_hooks)
import dbus


def install_hooks():
    """Install the dbus hooks into bzrlib."""
    Branch.hooks.install_named_hook(
        'post_change_branch_tip', on_post_change_branch_tip,
        'Announcing on branch change on D-Bus')
    install_mainloop_hooks()
    MainloopThread.hooks.install_named_hook(
        'server_started', on_server_start,
        'Registering server URL mapping')
    MainloopThread.hooks.install_named_hook(
        'server_stopped', on_server_stop,
        'Unregistering server mapping')


def get_activity():
    try:
        return activity.Activity()
    except dbus.DBusException, e:
        warning("Unable to contact DBus session: " + str(e))
        return None


def on_post_change_branch_tip(params):
    """Announce the new head revision of the branch to dbus."""
    activity = get_activity()
    if activity is None:
        return
    activity.advertise_branch(params.branch)


_announcer = None

def get_announcer():
    global _announcer
    if _announcer is None:
        _announcer = activity.ServerAnnouncer()
    return _announcer


def on_server_start(backing_urls, public_url):
    """Add the servers local and public urls to the session Broadcaster."""
    get_announcer().on_server_started(backing_urls, public_url)


def on_server_stop(local_urls, public_url):
    """The server has shutdown, so remove the servers local and public urls."""
    get_announcer().on_server_stopped(backing_urls, public_url)
